@@ -105,6 +105,20 @@ public:
         return _get() == nullptr;
     }
 
+    /**
+     * Simply returns the value reference itself. This exists only is to make
+     * the API compatible with other other reference types, which all offer the
+     * same method.
+     */
+    const ValueReference<T>& derefAsValue() const { return *this; }
+
+    /**
+     * Simply returns the value reference itself. This exists only is to make
+     * the API compatible with other other reference types, which all offer the
+     * same method.
+     */
+    ValueReference<T>& derefAsValue() { return *this; }
+
     /**
      * Returns a pointer to the referred value. The result may be null if the
      * instance does not refer to a valid value.
@@ -132,7 +146,7 @@ public:
 
         try {
             if ( auto ptr = std::get<T*>(_ptr) ) {
-                if constexpr ( std::is_base_of<Controllable<T>, T>::value )
+                if constexpr ( std::is_base_of_v<Controllable<T>, T> )
                     return ptr->shared_from_this();
                 else
                     throw IllegalReference("cannot dynamically create reference for type");
@@ -299,11 +313,11 @@ private:
      * stays around.
      */
     explicit ValueReference(T* t) : _ptr(t) {
-        static_assert(std::is_base_of<Controllable<T>, T>::value);
+        static_assert(std::is_base_of_v<Controllable<T>, T>);
         assert(t);
     }
 
-    inline const T* _get() const noexcept {
+    const T* _get() const noexcept {
         if ( auto ptr = std::get_if<T*>(&_ptr) )
             return *ptr;
 
@@ -311,7 +325,7 @@ private:
         return std::get_if<std::shared_ptr<T>>(&_ptr)->get();
     }
 
-    inline T* _get() noexcept {
+    T* _get() noexcept {
         if ( auto ptr = std::get_if<T*>(&_ptr) )
             return *ptr;
 
@@ -319,7 +333,7 @@ private:
         return std::get_if<std::shared_ptr<T>>(&_ptr)->get();
     }
 
-    inline const T* _safeGet() const {
+    const T* _safeGet() const {
         assert(_ptr.index() != std::variant_npos);
 
         // If the reference contains a raw pointer it is never null.
@@ -332,7 +346,7 @@ private:
         reference::detail::throw_null();
     }
 
-    inline T* _safeGet() {
+    T* _safeGet() {
         assert(_ptr.index() != std::variant_npos);
 
         // If the reference contains a raw pointer it is never null.
@@ -702,6 +716,23 @@ public:
         }
     }
 
+    /**
+     * Returns a value reference that is linked to the referred value. If the
+     * strong reference is null or expired, the returned reference will be
+     * null.
+     */
+    template<typename T>
+    ValueReference<T> derefAsValue() const {
+        if ( _ptr.empty() )
+            return {};
+
+        try {
+            return hilti::rt::any_cast<StrongReference<T>>(_ptr).derefAsValue();
+        } catch ( const hilti::rt::bad_any_cast& ) {
+            throw IllegalReference("invalid target type");
+        }
+    }
+
     /** Releases the bound reference. */
     void reset() { _ptr.clear(); }
 
