@@ -5,16 +5,13 @@
 #include <algorithm>
 #include <array>
 #include <cassert>
-#include <cinttypes>
 #include <cstddef>
 #include <cstring>
-#include <limits>
 #include <memory>
 #include <optional>
 #include <string>
 #include <tuple>
 #include <utility>
-#include <variant>
 #include <vector>
 
 #include <hilti/rt/any.h>
@@ -35,10 +32,38 @@ class Stream;
 namespace stream {
 class View;
 class SafeConstIterator;
+struct NonOwning {};
 
 namespace detail {
 class UnsafeConstIterator;
 }
+
+/**
+ * Accumulates statistics about a stream's data over its lifetime. Empty chunks
+ * are not counted.
+ */
+struct Statistics {
+    uint64_t num_data_bytes = 0;
+    uint64_t num_data_chunks = 0;
+    uint64_t num_gap_bytes = 0;
+    uint64_t num_gap_chunks = 0;
+
+    friend bool operator==(const Statistics& a, const Statistics& b) {
+        return a.num_data_bytes == b.num_data_bytes && a.num_data_chunks == b.num_data_chunks &&
+               a.num_gap_bytes == b.num_gap_bytes && a.num_gap_chunks == b.num_gap_chunks;
+    }
+
+    friend bool operator!=(const Statistics& a, const Statistics& b) { return ! (a == b); }
+
+    Statistics& operator+=(Statistics& other) {
+        num_data_bytes += other.num_data_bytes;
+        num_data_chunks += other.num_data_chunks;
+        num_gap_bytes += other.num_gap_bytes;
+        num_gap_chunks += other.num_gap_chunks;
+        return *this;
+    }
+};
+
 } // namespace stream
 
 namespace detail::adl {
@@ -46,6 +71,7 @@ std::string to_string(const Stream& x, adl::tag /*unused*/);
 std::string to_string(const stream::View& x, adl::tag /*unused*/);
 std::string to_string(const stream::SafeConstIterator& x, adl::tag /*unused*/);
 std::string to_string(const stream::detail::UnsafeConstIterator& x, adl::tag /*unused*/);
+std::string to_string(const stream::Statistics& x, adl::tag /*unused*/);
 } // namespace detail::adl
 
 namespace stream {
@@ -77,9 +103,10 @@ struct Gap {
  * stream's *Chain* links multiple of these chunks to represent all of its
  * content.
  *
- * A chunk internally employs small-buffer optimization for very small
- * amounts of data, storing it directly inside the instance instead of using
- * heap-allocated memory.
+ * A chunk may or may not own its data. The former is the default for
+ * construction and extension, unless specified explicitly otherwise. When
+ * non-owning, the creator needs to ensure the data stays around as long as the
+ * chunk does.
  *
  * All public methods of Chunk are constant. Modifications can be done only
  * be through the owning Chain (so that we can track changes there).
@@ -87,37 +114,85 @@ struct Gap {
 class Chunk {
 public:
     Chunk(const Offset& o, const View& d);
-    Chunk(const Offset& o, std::string s);
+    Chunk(const Offset& o, std::string_view s);
+
+    // Constructs a chunk that own its data by copying what's passed in.
+    Chunk(const Offset& o, const Byte* b, size_t size);
+
+    // Constructs a chunk that does not own its data.
+    Chunk(const Offset& o, const Byte* b, size_t size, NonOwning) : _offset(o), _size(size), _data(b) {}
 
     // Constructs a gap chunk which signifies empty data.
-    Chunk(const Offset& o, size_t len) : _offset(o), _gap_size(len) { assert(_gap_size > 0); }
+    Chunk(const Offset& o, size_t len) : _offset(o), _size(len) { assert(_size > 0); }
+
+    Chunk(const Chunk& other)
+        : _offset(other._offset), _size(other._size), _data(other._data), _chain(other._chain), _next(nullptr) {
+        if ( other.isOwning() )
+            makeOwning();
+    }
 
-    Chunk(const Chunk& other) : _offset(other._offset), _data(other._data) {}
     Chunk(Chunk&& other) noexcept
-        : _offset(other._offset), _data(std::move(other._data)), _next(std::move(other._next)) {}
+        : _offset(other._offset),
+          _size(other._size),
+          _allocated(other._allocated),
+          _data(other._data),
+          _chain(other._chain),
+          _next(std::move(other._next)) {
+        other._size = 0;
+        other._allocated = 0;
+        other._data = nullptr;
+    }
 
-    Chunk& operator=(const Chunk& other) = delete;
+    Chunk& operator=(const Chunk& other) {
+        if ( &other == this )
+            return *this;
+
+        destroy();
 
-    Chunk& operator=(Chunk&& other) noexcept {
         _offset = other._offset;
-        _data = std::move(other._data);
-        _next = std::move(other._next);
+        _size = other._size;
+        _data = other._data;
+        _allocated = 0;
         _chain = other._chain;
+        _next = nullptr;
+
+        if ( other.isOwning() )
+            makeOwning();
+
         return *this;
     }
 
-    ~Chunk();
+    Chunk& operator=(Chunk&& other) noexcept {
+        if ( _allocated > 0 )
+            delete[] _data;
+
+        _offset = other._offset;
+        _size = other._size;
+        _allocated = other._allocated;
+        _data = other._data;
+        _chain = other._chain;
+        _next = std::move(other._next);
+
+        other._size = 0;
+        other._allocated = 0;
+        other._data = nullptr;
+
+        return *this;
+    }
+
+    ~Chunk() { destroy(); }
 
     Offset offset() const { return _offset; }
     Offset endOffset() const { return _offset + size(); }
-    bool isGap() const { return _gap_size > 0; };
+    bool isGap() const { return _data == nullptr; };
+    bool isOwning() const { return _allocated > 0; }
     bool inRange(const Offset& offset) const { return offset >= _offset && offset < endOffset(); }
 
     const Byte* data() const {
         if ( isGap() )
             throw MissingData("data is missing");
 
-        return reinterpret_cast<const Byte*>(_data.data());
+        return _data;
     }
 
     const Byte* data(const Offset& offset) const {
@@ -129,15 +204,11 @@ public:
         if ( isGap() )
             throw MissingData("data is missing");
 
-        return reinterpret_cast<const Byte*>(data() + _data.size());
+        return data() + _size;
     }
 
-    Size size() const {
-        if ( isGap() )
-            return _gap_size;
-
-        return _data.size();
-    }
+    Size size() const { return _size; }
+    Size allocated() const { return _allocated; }
 
     bool isLast() const { return ! _next; }
     const Chunk* next() const { return _next.get(); }
@@ -156,6 +227,18 @@ public:
         return i;
     }
 
+    // Creates a new copy of the data internally if the chunk is currently not
+    // owning it. On return, is guaranteed to now own the data.
+    void makeOwning() {
+        if ( _size == 0 || _allocated > 0 || ! _data )
+            return;
+
+        auto data = std::make_unique<Byte[]>(_size);
+        memcpy(data.get(), _data, _size);
+        _allocated = _size;
+        _data = data.release();
+    }
+
     void debugPrint(std::ostream& out) const;
 
 protected:
@@ -184,11 +267,13 @@ protected:
 
     Chunk* next() { return _next.get(); }
 
-    // Link in chunk as successor of current one. Updates offset/chain for
-    // the appended chunk and all its successors.
+    // Link in chunk as successor of current one. Updates offset/chain for the
+    // appended chunk and all its successors. Makes the current chunk owning,
+    // so that at most the last chunk in a chain can be non-owning.
     void setNext(std::unique_ptr<Chunk> next) {
         assert(_chain);
 
+        makeOwning();
         Offset offset = endOffset();
         _next = std::move(next);
 
@@ -201,12 +286,26 @@ protected:
         }
     }
 
-    void clearNext() { _next = nullptr; }
+    // Reset chunk state to no longer reference a chain. Note that this does
+    // not update its predecessor inside the chain if that exists.
+    void detach() {
+        _offset = 0;
+        _chain = nullptr;
+        _next = nullptr;
+    }
 
 private:
+    Chunk() {}
+
+    // Deletes all allocated/owned data, safely. Members will be left in
+    // undefined state afterwards and need re-initialization if instance
+    // remains in use.
+    void destroy();
+
     Offset _offset = 0;            // global offset of 1st byte
-    std::string _data;             // content of this chunk
-    size_t _gap_size = 0;          // non-zero if this is a gap in which case _data is irrelevant
+    size_t _size = 0;              // size of payload or gap
+    size_t _allocated = 0;         // size of memory allocated for data, which can be more than its size
+    const Byte* _data = nullptr;   // chunk's payload, or null for gap chunks
     const Chain* _chain = nullptr; // chain this chunk is part of, or null if not linked to a chain yet (non-owning;
                                    // will stay valid at least as long as the current chunk does)
     std::unique_ptr<Chunk> _next = nullptr; // next chunk in chain, or null if last
@@ -227,7 +326,20 @@ public:
     Chain() {}
 
     /** Moves a chunk and all its successors into a new chain. */
-    Chain(std::unique_ptr<Chunk> head) : _head(std::move(head)), _tail(_head->last()) { _head->setChain(this); }
+    Chain(std::unique_ptr<Chunk> head) : _head(std::move(head)), _tail(_head->last()) {
+        _head->setChain(this);
+
+        if ( auto size = _head->size() ) {
+            if ( _head->isGap() ) {
+                _statistics.num_gap_bytes = _head->size();
+                _statistics.num_gap_chunks = 1;
+            }
+            else {
+                _statistics.num_data_bytes = _head->size();
+                _statistics.num_data_chunks = 1;
+            }
+        }
+    }
 
     Chain(Chain&& other) = delete;
     Chain(const Chain& other) = delete;
@@ -236,6 +348,7 @@ public:
 
     const Chunk* head() const { return _head.get(); }
     const Chunk* tail() const { return _tail; }
+    Chunk* tail() { return _tail; }
     Size size() const { return (endOffset() - offset()).Ref(); }
     bool isFrozen() const { return _state == State::Frozen; }
     bool isValid() const { return _state != State::Invalid; }
@@ -262,13 +375,27 @@ public:
     UnsafeConstIterator unsafeBegin() const;
     UnsafeConstIterator unsafeEnd() const;
 
-    // Returns a newly allocated chain with the same content.
-    ChainPtr deepCopy() const;
+    // Returns a newly allocated chain with the same content and statistics.
+    ChainPtr copy() const;
 
-    /** Appends a new chunk to the end. */
-    void append(std::unique_ptr<Chunk> chunk);
+    // Appends a new chunk to the end, copying the data.
+    void append(const Byte* data, size_t size);
+
+    // Appends a new chunk to the end, not copying the data.
+    void append(const Byte* data, size_t size, stream::NonOwning);
+
+    // Appends a new chunk to the end, moving the data.
+    void append(Bytes&& data);
+
+    // Appends another chain to the end.
     void append(Chain&& other);
 
+    // Appends a new chunk to the end.
+    void append(std::unique_ptr<Chunk> chunk);
+
+    // Appends a gap to the end.
+    void appendGap(size_t size);
+
     void trim(const Offset& offset);
     void trim(const SafeConstIterator& i);
     void trim(const UnsafeConstIterator& i);
@@ -280,6 +407,7 @@ public:
         _head.reset();
         _head_offset = 0;
         _tail = nullptr;
+        _statistics = {};
     }
 
     // Turns the chain into a freshly initialized state.
@@ -288,12 +416,14 @@ public:
         _head.reset();
         _head_offset = 0;
         _tail = nullptr;
+        _statistics = {};
     }
 
     void freeze() {
         if ( isValid() )
             _state = State::Frozen;
     }
+
     void unfreeze() {
         if ( isValid() )
             _state = State::Mutable;
@@ -302,6 +432,10 @@ public:
     // Returns the number of dynamic chunks allocated.
     int numberOfChunks() const;
 
+    // Returns statistics for the chain. These are accumulative over the whole
+    // lifetime of the chain.
+    const auto& statistics() const { return _statistics; }
+
 private:
     void _ensureValid() const {
         if ( ! isValid() )
@@ -334,6 +468,11 @@ private:
     // Always pointing to last chunk reachable from *head*, or null if chain
     // is empty; non-owning
     Chunk* _tail = nullptr;
+
+    // Tracks statistics as new data comes in.
+    stream::Statistics _statistics;
+
+    std::unique_ptr<Chunk> _cached; // previously freed chunk for reuse
 };
 
 } // namespace detail
@@ -1227,9 +1366,9 @@ public:
     }
 
     /**
-     * Advances the view's starting position by a given number of stream.
+     * Advances the view's starting position by a given number of stream bytes.
      *
-     * @param i the number of stream to advance.
+     * @param i the number of stream bytes to advance.
      */
     View advance(const integer::safe<uint64_t>& i) const { return View(begin() + i, _end); }
 
@@ -1366,12 +1505,18 @@ public:
     /** Returns an safe iterator pointint to the beginning of the view. */
     const SafeConstIterator& begin() const { return _begin; }
 
+    /** Same as `begin()`, just for compatibility with std types. */
+    const SafeConstIterator& cbegin() const { return _begin; }
+
     /** Returns a safe iterator representing the end of the instance. */
     SafeConstIterator end() const {
         assert(_begin.chain());
         return _end ? *_end : _begin.chain()->end();
     }
 
+    /** Same as `end()`, just for compatibility with std types. */
+    SafeConstIterator cend() const { return end(); }
+
     /** State for block-wise iteration of a stream instance. */
     struct Block {
         const Byte* start;           /**< Pointer to first byte. */
@@ -1423,11 +1568,11 @@ private:
     }
 
     void _ensureValid() const {
-        //        if ( ! _begin.chain() )
-        //            throw InvalidIterator("view has invalid beginning");
+        if ( ! _begin.isValid() )
+            throw InvalidIterator("view has invalid beginning");
 
-        //        if ( ! _begin.isValid() )
-        //            throw InvalidIterator("view has invalid beginning");
+        if ( (! _begin.isUnset()) && _begin.offset() < _begin.chain()->offset() )
+            throw InvalidIterator("view starts before available range");
 
         if ( _end && ! _end->isValid() )
             throw InvalidIterator("view has invalid end");
@@ -1511,7 +1656,15 @@ public:
      * Creates an instance from an existing memory block. The data
      * will be copied if set, otherwise a gap will be recorded.
      */
-    Stream(const char* d, const Size& n);
+    Stream(const char* d, Size n) : Stream() { append(d, n); }
+
+    /**
+     * Creates an instance from an existing memory block. The data will not be
+     * copied and hence must remain valid until the stream ether is destroyed
+     * or `makeOwning()` gets called, whatever comes first. Passing a nullptr
+     * for the data records a gap.
+     */
+    Stream(const char* d, Size n, stream::NonOwning) : Stream() { append(d, n, stream::NonOwning()); }
 
     /**
      * Creates an instance from an existing stream view.
@@ -1530,7 +1683,7 @@ public:
      * Constructs a stream from another stream instance.
      * @param other instance to create this stream from
      */
-    Stream(const Stream& other) : _chain(other._chain->deepCopy()) {}
+    Stream(const Stream& other) : _chain(other._chain->copy()) {}
 
     /**
      * Constructs a stream from another stream instance.
@@ -1561,7 +1714,7 @@ public:
             return *this;
 
         _chain->invalidate();
-        _chain = other._chain->deepCopy();
+        _chain = other._chain->copy();
         return *this;
     }
 
@@ -1595,12 +1748,26 @@ public:
      */
     void append(std::unique_ptr<const Byte*> data);
 
-    /** Appends the content of a raw memory area, copying the data. This function does not invalidate iterators.
+    /**
+     * Appends the content of a raw memory area, copying the data. This
+     * function does not invalidate iterators.
      * @param data pointer to the data to append. If this is nullptr and gap will be appended instead.
      * @param len length of the data to append
      */
     void append(const char* data, size_t len);
 
+
+    /**
+     * Appends the content of a raw memory area, *not* copying the data. This
+     * function does not invalidate iterators. Because the data will not be
+     * copied, it must remain valid until the stream is either destroyed or
+     * `makeOwning()` gets called.
+     *
+     * @param data pointer to the data to append. If this is nullptr and gap will be appended instead.
+     * @param len length of the data to append
+     */
+    void append(const char* data, size_t len, stream::NonOwning);
+
     /**
      * Cuts off the beginning of the data up to, but excluding, a given
      * iterator. All existing iterators pointing beyond that point will
@@ -1620,11 +1787,28 @@ public:
     /** Returns true if the instance is currently frozen. */
     bool isFrozen() const { return _chain->isFrozen(); }
 
+    /**
+     * Returns the stream into a freshly initialized state, as if it was just
+     * created. (This concerns only externally visible state, it retains any
+     * potentially cached resources for reuse.)
+     */
+    void reset() { _chain->reset(); }
+
+    /** Ensure the stream fully owns all its data. */
+    void makeOwning() {
+        // Only the final chunk can be non-owning, that's guaranteed by
+        // `Chunk::setNext()`.
+        if ( auto* t = _chain->tail() )
+            t->makeOwning();
+    }
+
     /** Returns a safe iterator representing the first byte of the instance. */
     SafeConstIterator begin() const { return _chain->begin(); }
+    SafeConstIterator cbegin() const { return begin(); }
 
     /** Returns a safe iterator representing the end of the instance. */
     SafeConstIterator end() const { return _chain->end(); }
+    SafeConstIterator cend() const { return end(); }
 
     /** Returns an unsafe iterator representing the first byte of the instance. */
     UnsafeConstIterator unsafeBegin() const { return _chain->unsafeBegin(); }
@@ -1662,6 +1846,12 @@ public:
      */
     int numberOfChunks() const { return _chain->numberOfChunks(); }
 
+    /*
+     * Returns statistics for the chain. These are accumulative over the whole
+     * lifetime of the chain.
+     */
+    const auto& statistics() const { return _chain->statistics(); }
+
     /**
      * Prints out a debug rendering to the stream's internal representation.
      */
@@ -1689,6 +1879,9 @@ inline std::string detail::to_string_for_print<Stream>(const Stream& x) {
 }
 
 inline std::ostream& operator<<(std::ostream& out, const Stream& x) { return out << to_string_for_print(x); }
+inline std::ostream& operator<<(std::ostream& out, const stream::Statistics& x) {
+    return out << to_string_for_print(x);
+}
 
 namespace detail::adl {
 inline std::string to_string(const stream::View& x, adl::tag /*unused*/) {
@@ -1696,6 +1889,12 @@ inline std::string to_string(const stream::View& x, adl::tag /*unused*/) {
 }
 
 inline std::string to_string(const Stream& x, adl::tag /*unused*/) { return hilti::rt::to_string(x.view()); }
-} // namespace detail::adl
+inline std::string to_string(const stream::Statistics& x, adl::tag /*unused*/) {
+    // Render like a struct.
+    return fmt("[$num_data_bytes=%" PRIu64 ", $num_data_chunks=%" PRIu64 ", $num_gap_bytes=%" PRIu64
+               ", $num_gap_chunks=%" PRIu64 "]",
+               x.num_data_bytes, x.num_data_chunks, x.num_gap_bytes, x.num_gap_chunks);
+}
 
+} // namespace detail::adl
 } // namespace hilti::rt
