@@ -5,13 +5,19 @@
 #include "zeek/IPAddr.h"
 #include "zeek/Reporter.h"
 #include "zeek/ZeekString.h"
+#include "zeek/script_opt/ZAM/BuiltIn.h"
 #include "zeek/script_opt/ZAM/Compile.h"
 
 namespace zeek::detail {
 
 const ZAMStmt ZAMCompiler::CompileStmt(const Stmt* s) {
-    curr_stmt = const_cast<Stmt*>(s)->ThisPtr();
-    ASSERT(curr_stmt->Tag() == STMT_NULL || curr_stmt->GetLocationInfo()->first_line != 0);
+    auto loc = s->GetLocationInfo();
+    ASSERT(loc->first_line != 0 || s->Tag() == STMT_NULL);
+    auto loc_copy =
+        std::make_shared<Location>(loc->filename, loc->first_line, loc->last_line, loc->first_column, loc->last_column);
+    ASSERT(! AST_blocks || s->Tag() == STMT_NULL || AST_blocks->HaveExpDesc(loc_copy.get()));
+    auto loc_parent = ZAM::curr_loc->Parent();
+    ZAM::curr_loc = std::make_shared<ZAMLocInfo>(ZAM::curr_func, std::move(loc_copy), ZAM::curr_loc->Parent());
 
     switch ( s->Tag() ) {
         case STMT_PRINT: return CompilePrint(static_cast<const PrintStmt*>(s));
@@ -22,10 +28,6 @@ const ZAMStmt ZAMCompiler::CompileStmt(const Stmt* s) {
 
         case STMT_SWITCH: return CompileSwitch(static_cast<const SwitchStmt*>(s));
 
-        case STMT_ADD: return CompileAdd(static_cast<const AddStmt*>(s));
-
-        case STMT_DELETE: return CompileDel(static_cast<const DelStmt*>(s));
-
         case STMT_EVENT: {
             auto es = static_cast<const EventStmt*>(s);
             auto e = static_cast<const EventExpr*>(es->StmtExpr());
@@ -76,7 +78,7 @@ const ZAMStmt ZAMCompiler::CompilePrint(const PrintStmt* ps) {
             return Print1C(e0->AsConstExpr());
     }
 
-    return PrintO(BuildVals(l));
+    return PrintO(BuildVals(l).get());
 }
 
 const ZAMStmt ZAMCompiler::CompileExpr(const ExprStmt* es) {
@@ -174,6 +176,36 @@ const ZAMStmt ZAMCompiler::IfElse(const Expr* e, const Stmt* s1, const Stmt* s2)
         case OP_HAS_FIELD_COND_VVV: z->op = OP_NOT_HAS_FIELD_COND_VVV; break;
         case OP_NOT_HAS_FIELD_COND_VVV: z->op = OP_HAS_FIELD_COND_VVV; break;
 
+        case OP_CONN_EXISTS_COND_VV: z->op = OP_NOT_CONN_EXISTS_COND_VV; break;
+        case OP_NOT_CONN_EXISTS_COND_VV: z->op = OP_CONN_EXISTS_COND_VV; break;
+
+        case OP_IS_ICMP_PORT_COND_VV: z->op = OP_NOT_IS_ICMP_PORT_COND_VV; break;
+        case OP_NOT_IS_ICMP_PORT_COND_VV: z->op = OP_IS_ICMP_PORT_COND_VV; break;
+
+        case OP_IS_TCP_PORT_COND_VV: z->op = OP_NOT_IS_TCP_PORT_COND_VV; break;
+        case OP_NOT_IS_TCP_PORT_COND_VV: z->op = OP_IS_TCP_PORT_COND_VV; break;
+
+        case OP_IS_UDP_PORT_COND_VV: z->op = OP_NOT_IS_UDP_PORT_COND_VV; break;
+        case OP_NOT_IS_UDP_PORT_COND_VV: z->op = OP_IS_UDP_PORT_COND_VV; break;
+
+        case OP_IS_V4_ADDR_COND_VV: z->op = OP_NOT_IS_V4_ADDR_COND_VV; break;
+        case OP_NOT_IS_V4_ADDR_COND_VV: z->op = OP_IS_V4_ADDR_COND_VV; break;
+
+        case OP_IS_V6_ADDR_COND_VV: z->op = OP_NOT_IS_V6_ADDR_COND_VV; break;
+        case OP_NOT_IS_V6_ADDR_COND_VV: z->op = OP_IS_V6_ADDR_COND_VV; break;
+
+        case OP_READING_LIVE_TRAFFIC_COND_V: z->op = OP_NOT_READING_LIVE_TRAFFIC_COND_V; break;
+        case OP_NOT_READING_LIVE_TRAFFIC_COND_V: z->op = OP_READING_LIVE_TRAFFIC_COND_V; break;
+
+        case OP_READING_TRACES_COND_V: z->op = OP_NOT_READING_TRACES_COND_V; break;
+        case OP_NOT_READING_TRACES_COND_V: z->op = OP_READING_TRACES_COND_V; break;
+
+        case OP_TABLE_HAS_ELEMENTS_COND_VV: z->op = OP_NOT_TABLE_HAS_ELEMENTS_COND_VV; break;
+        case OP_NOT_TABLE_HAS_ELEMENTS_COND_VV: z->op = OP_TABLE_HAS_ELEMENTS_COND_VV; break;
+
+        case OP_VECTOR_HAS_ELEMENTS_COND_VV: z->op = OP_NOT_VECTOR_HAS_ELEMENTS_COND_VV; break;
+        case OP_NOT_VECTOR_HAS_ELEMENTS_COND_VV: z->op = OP_VECTOR_HAS_ELEMENTS_COND_VV; break;
+
         case OP_VAL_IS_IN_TABLE_COND_VVV: z->op = OP_VAL_IS_NOT_IN_TABLE_COND_VVV; break;
         case OP_VAL_IS_NOT_IN_TABLE_COND_VVV: z->op = OP_VAL_IS_IN_TABLE_COND_VVV; break;
 
@@ -200,10 +232,6 @@ const ZAMStmt ZAMCompiler::GenCond(const Expr* e, int& branch_v) {
     auto op1 = e->GetOp1();
     auto op2 = e->GetOp2();
 
-    NameExpr* n1 = nullptr;
-    NameExpr* n2 = nullptr;
-    ConstExpr* c = nullptr;
-
     if ( e->Tag() == EXPR_HAS_FIELD ) {
         auto hf = e->AsHasFieldExpr();
         auto z = GenInst(OP_HAS_FIELD_COND_VVV, op1->AsNameExpr(), hf->Field());
@@ -213,7 +241,6 @@ const ZAMStmt ZAMCompiler::GenCond(const Expr* e, int& branch_v) {
     }
 
     if ( e->Tag() == EXPR_IN ) {
-        auto op1 = e->GetOp1();
         auto op2 = e->GetOp2()->AsNameExpr();
 
         // First, deal with the easy cases: it's a single index.
@@ -288,6 +315,31 @@ const ZAMStmt ZAMCompiler::GenCond(const Expr* e, int& branch_v) {
         return AddInst(z);
     }
 
+    if ( e->Tag() == EXPR_CALL ) {
+        auto c = static_cast<const CallExpr*>(e);
+        if ( IsZAM_BuiltInCond(this, c, branch_v) )
+            return LastInst();
+    }
+
+    if ( e->Tag() == EXPR_SCRIPT_OPT_BUILTIN ) {
+        auto bi = static_cast<const ScriptOptBuiltinExpr*>(e);
+        ASSERT(bi->Tag() == ScriptOptBuiltinExpr::HAS_ELEMENTS);
+        auto aggr = bi->GetOp1()->AsNameExpr();
+
+        ZOp op;
+        if ( aggr->GetType()->Tag() == TYPE_TABLE )
+            op = OP_TABLE_HAS_ELEMENTS_COND_VV;
+        else
+            op = OP_VECTOR_HAS_ELEMENTS_COND_VV;
+
+        branch_v = 2;
+        return AddInst(GenInst(op, aggr, +0));
+    }
+
+    NameExpr* n1 = nullptr;
+    NameExpr* n2 = nullptr;
+    ConstExpr* c = nullptr;
+
     if ( op1->Tag() == EXPR_NAME ) {
         n1 = op1->AsNameExpr();
 
@@ -567,46 +619,6 @@ const ZAMStmt ZAMCompiler::TypeSwitch(const SwitchStmt* sw, const NameExpr* v, c
     return body_end;
 }
 
-const ZAMStmt ZAMCompiler::CompileAdd(const AddStmt* as) {
-    auto e = as->StmtExprPtr();
-    auto aggr = e->GetOp1()->AsNameExpr();
-    auto index_list = e->GetOp2();
-
-    if ( index_list->Tag() != EXPR_LIST )
-        reporter->InternalError("non-list in \"add\"");
-
-    auto indices = index_list->AsListExprPtr();
-    auto& exprs = indices->Exprs();
-
-    if ( exprs.length() == 1 ) {
-        auto e1 = exprs[0];
-        if ( e1->Tag() == EXPR_NAME )
-            return AddStmt1VV(aggr, e1->AsNameExpr());
-        else
-            return AddStmt1VC(aggr, e1->AsConstExpr());
-    }
-
-    return AddStmtVO(aggr, BuildVals(indices));
-}
-
-const ZAMStmt ZAMCompiler::CompileDel(const DelStmt* ds) {
-    auto e = ds->StmtExprPtr();
-    auto aggr = e->GetOp1()->AsNameExpr();
-
-    if ( e->Tag() == EXPR_FIELD ) {
-        int field = e->AsFieldExpr()->Field();
-        return DelFieldVi(aggr, field);
-    }
-
-    auto index_list = e->GetOp2();
-
-    if ( index_list->Tag() != EXPR_LIST )
-        reporter->InternalError("non-list in \"delete\"");
-
-    auto internal_ind = std::unique_ptr<OpaqueVals>(BuildVals(index_list->AsListExprPtr()));
-    return DelTableVO(aggr, internal_ind.get());
-}
-
 const ZAMStmt ZAMCompiler::CompileWhile(const WhileStmt* ws) {
     auto loop_condition = ws->Condition();
 
@@ -699,8 +711,8 @@ const ZAMStmt ZAMCompiler::LoopOverTable(const ForStmt* f, const NameExpr* val)
         int slot = id->IsBlank() ? -1 : FrameSlot(id);
         aux->loop_vars.push_back(slot);
         auto& t = id->GetType();
-        aux->loop_var_types.push_back(t);
-        aux->lvt_is_managed.push_back(ZVal::IsManagedType(t));
+        aux->types.push_back(t);
+        aux->is_managed.push_back(ZVal::IsManagedType(t));
     }
 
     bool no_loop_vars = (num_unused == loop_vars->length());
@@ -892,6 +904,16 @@ const ZAMStmt ZAMCompiler::CompileReturn(const ReturnStmt* r) {
 const ZAMStmt ZAMCompiler::CompileCatchReturn(const CatchReturnStmt* cr) {
     retvars.push_back(cr->RetVar());
 
+    auto hold_func = ZAM::curr_func;
+    auto hold_loc = ZAM::curr_loc;
+
+    ZAM::curr_func = cr->Func()->Name();
+
+    bool is_event_inline = (hold_func == ZAM::curr_func);
+
+    if ( ! is_event_inline )
+        ZAM::curr_loc = std::make_shared<ZAMLocInfo>(ZAM::curr_func, ZAM::curr_loc->LocPtr(), hold_loc);
+
     PushCatchReturns();
 
     auto block = cr->Block();
@@ -900,6 +922,14 @@ const ZAMStmt ZAMCompiler::CompileCatchReturn(const CatchReturnStmt* cr) {
 
     ResolveCatchReturns(GoToTargetBeyond(block_end));
 
+    if ( ! is_event_inline ) {
+        // Strictly speaking, we could do this even if is_event_inline
+        // is true, because the values won't have changed. However, that
+        // just looks weird, so we condition this to match the above.
+        ZAM::curr_func = hold_func;
+        ZAM::curr_loc = hold_loc;
+    }
+
     return block_end;
 }
 
@@ -998,7 +1028,8 @@ const ZAMStmt ZAMCompiler::InitVector(IDPtr id, VectorType* vt) {
 const ZAMStmt ZAMCompiler::InitTable(IDPtr id, TableType* tt, Attributes* attrs) {
     auto z = ZInstI(OP_INIT_TABLE_V, FrameSlot(id));
     z.SetType({NewRef{}, tt});
-    z.attrs = {NewRef{}, attrs};
+    z.aux = new ZInstAux(0);
+    z.aux->attrs = {NewRef{}, attrs};
     return AddInst(z);
 }
 
