@@ -75,10 +75,8 @@ public:
      * and to index it, inside the Zeekygen documentation.
      * @param description textual description in reST that will be shown for
      * this module inside the Zeekygen documentation
-     * @param mtime timestamp indicating last time of modification of any of
-     * the module's content; used by Zeekygen to trigger rebuilds as necessary
      */
-    void registerSpicyModuleBegin(const std::string& name, const std::string& description, hilti::rt::Time mtime);
+    void registerSpicyModuleBegin(const std::string& name, const std::string& description);
 
     /**
      * Runtime method to register a protocol analyzer with its Zeek-side
@@ -266,9 +264,6 @@ public:
      * loaded will also be activated. By calling this method, an analyzer can
      * toggled.
      *
-     * @note This is currently not supported because Zeek does not provide the
-     * necessary API.
-     *
      * @param analyzer tag of analyzer
      * @param enable true to enable, false to disable
      */
@@ -354,7 +349,7 @@ private:
         // Computed and available once the analyzer has been registered.
         std::string name_zeek;
         std::string name_zeekygen;
-        Tag::type_t type;
+        Tag tag;
         const ::spicy::rt::Parser* parser_orig;
         const ::spicy::rt::Parser* parser_resp;
         Tag replaces;
@@ -380,7 +375,7 @@ private:
         // Computed and available once the analyzer has been registered.
         std::string name_zeek;
         std::string name_zeekygen;
-        Tag::type_t type;
+        Tag tag;
         const ::spicy::rt::Parser* parser;
         Tag replaces;
 
@@ -404,7 +399,7 @@ private:
         // Computed and available once the analyzer has been registered.
         std::string name_zeek;
         std::string name_zeekygen;
-        Tag::type_t type;
+        Tag tag;
         const ::spicy::rt::Parser* parser;
         Tag replaces;
 
